﻿using System;
using System.Diagnostics.Contracts;

namespace codeContractsBook
{
     public class Book
    {
        public String Title { get; set; }
        public double Price { get; set; }

        public Book(string title, double price = 0.0)
        {
            Title = title;
        }

        [ContractInvariantMethod]
        private void ObjectInvariant()
        {
            Contract.Invariant(Title!= null && Title.Length >= 2);
            Contract.Invariant(Price >= 0.0);
        }

        public void RaisePriceTo(double newPrice)
        {
            Contract.Requires(newPrice > Price);
            Contract.Ensures(Price > Contract.OldValue(Price));
            Price = newPrice;
        }

        public void LowerPriceTo(double newPrice)
        {
            Contract.Requires<ArgumentOutOfRangeException>(newPrice < Price);
            Contract.EnsuresOnThrow<InvalidOperationException>(Price < Contract.OldValue(Price));
            Price = newPrice;
        }
    }
}
